@@ -159,11 +159,32 @@ static int Oops_write_bfd_data(const bfd *ibfd, bfd *obfd,
 static char *Oops_code_to_file(const char *code, int size, const bfd *ibfd,
 			       OPTIONS *options)
 {
-    char *file;
+    char *file, *tmpdir;
+    int fd;
     bfd *obfd;
     const bfd_arch_info_type *bai;
+    static const char temp_suffix[] = "/ksymoops.XXXXXX";
+    static char const procname[] = "Oops_code_to_file";
 
-    file = tmpnam(NULL);
+    /* Security fix, use mkstemp and honour TMPDIR */
+    if (!(tmpdir = getenv("TMPDIR")) || !*tmpdir) {
+#ifdef P_tmpdir
+	tmpdir = P_tmpdir;
+#else
+	tmpdir = "/tmp";
+#endif
+    }
+    file = malloc(strlen(tmpdir) + sizeof(temp_suffix));
+    if (!file)
+	malloc_error(procname);
+    strcpy(file, tmpdir);
+    strcat(file, temp_suffix);
+    if ((fd = mkstemp(file)) < 0) {
+	ERROR("Unable to open mkstemp file '%s'\n", file);
+	perror(prefix);
+	return(NULL);
+    }
+    close(fd);
 
     /* Set the target on the output file */
     if (!(obfd = bfd_openw(file, options->target))) {
@@ -311,13 +332,13 @@ static void Oops_decode_one(SYMBOL_SET *ss, const char *line, addr_t eip,
  * this proc cares who created the Code: line.
  */
 static int Oops_code_values(const unsigned char* code_text, unsigned char *code,
-			    int *adjust, char ***string, int string_max,
-			    const OPTIONS *options)
+			    int *adjust, const OPTIONS *options)
 {
-    int byte = 0, byte_prev, len;
+    int byte = 0, byte_prev, len, ret = 1;
     U16 u16;
     U32 u32;
     U64 u64, value;
+    char **string = NULL;
     const char *p;
     static regex_t     re_Oops_code_value;
     static regmatch_t *re_Oops_code_value_pmatch;
@@ -342,38 +363,44 @@ static int Oops_code_values(const unsigned char* code_text, unsigned char *code,
      */
     RE_COMPILE(&re_Oops_code_value,
 	    "^"
-	    "([<(]?)"			/* 1 */
-	    "([0-9a-fA-F]+)"		/* 2 */
-	    "[)>]?"
-	    " *"
+	    "("				/* 1 */
+	      "([<(]?)"			/* 2 */
+	      "([0-9a-fA-F]+)"		/* 3 */
+	      "[)>]?"
+	      " *"
+	    ")"
+	    "|(Bad EIP value)"		/* 4 */
 	    ,
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_code_value_pmatch);
 
-    re_string_check(re_Oops_code_value.re_nsub+1, string_max, procname);
     while (regexec(&re_Oops_code_value, p, re_Oops_code_value.re_nsub+1,
 	    re_Oops_code_value_pmatch, 0) == 0) {
 	re_strings(&re_Oops_code_value, p,
-	    re_Oops_code_value_pmatch, string);
+	    re_Oops_code_value_pmatch, &string);
+	if (string[4] && *(string[4])) {
+	    ret = 0;
+	    break;	/* Bad EIP value, no code bytes */
+	}
 	if (byte >= CODE_SIZE)
 	    break;
-	value = hexstring((*string)[2]);
+	value = hexstring(string[3]);
 	if (errno) {
 	    ERROR("Invalid hex value in code_value line, treated as zero - "
-	          "'%s'\ncode_value line '%s'", (*string)[2], code_text);
+	          "'%s'\ncode_value line '%s'", string[3], code_text);
 	    perror(prefix);
 	    value = 0;
 	}
-	if ((*string)[1] && *((*string)[1]))
+	if (string[2] && *(string[2]))
 	    *adjust = -byte;    /* this byte is EIP */
 	/* On some architectures Code: is a stream of bytes, on some it is a
 	 * stream of shorts, on some it is a stream of ints.  Consistent we're
 	 * not!
 	 */
-	len = strlen((*string)[2]);
+	len = strlen(string[3]);
 	byte_prev = byte;
 	if (byte+len/2 > CODE_SIZE) {
-	    WARNING("extra values in Code line, ignored - '%s'", (*string)[2]);
+	    WARNING("extra values in Code line, ignored - '%s'", string[3]);
 	    break;
 	}
 	switch (len) {
@@ -397,7 +424,7 @@ static int Oops_code_values(const unsigned char* code_text, unsigned char *code,
 		break;
 	    default:
 		ERROR("invalid value 0x%s in Code line, must be 2, 4, 8 or 16 "
-		      "digits, value ignored", (*string)[2]);
+		      "digits, value ignored", string[3]);
 		break;
 	}
 	if (options->endianess && len != 2) {
@@ -415,10 +442,11 @@ static int Oops_code_values(const unsigned char* code_text, unsigned char *code,
 	p += re_Oops_code_value_pmatch[0].rm_eo;
     }
 
-    if (*p)
+    if (ret && *p)
 	WARNING("garbage '%s' at end of code line ignored", p);
 
-    return(1);
+    re_strings_free(&re_Oops_code_value, &string);
+    return(ret);
 }
 
 /* Add an objdump line to the symbol table.  Just to keep the non-ix86 users
@@ -492,16 +520,21 @@ static void Oops_set_eip(const char *value, addr_t *eip, SYMBOL_SET *ss,
 
 static regex_t     re_Oops_i370_regs;
 static regmatch_t *re_Oops_i370_regs_pmatch;
+static int         s390_reg_num = -1;
 
 /* All address arithmetic is done in 64 bit mode, truncate to the result to
  * a valid range for the target.
  */
 static addr_t Oops_truncate_address(addr_t address, const OPTIONS *options)
 {
-    if (options->address_bits == 32 || strstr(options->target, "i370")) {
+    if (options->address_bits == 32 ||
+        (options->target && (
+	 strstr(options->target, "i370") ||
+         strstr(options->target, "s390")))) {
 	U32 u32 = address;
-	if (strstr(options->target, "i370"))
-	    u32 &= 0x7fffffff;  /* Really 31 bit addressing on i370 */
+	if (strstr(options->target, "i370") ||
+	    strstr(options->target, "s390"))
+	    u32 &= 0x7fffffff;  /* Really 31 bit addressing on i370/s390 */
 	return(u32);
     }
     return(address);	/* 0, 64, anything else */
@@ -527,7 +560,8 @@ static const char *Oops_arch_to_eip(const OPTIONS *options)
     }
     if (strstr(options->target, "powerpc"))
 	return("NIP");
-    if (strstr(options->target, "i370"))
+    if (strstr(options->target, "i370") ||
+        strstr(options->target, "s390"))
 	return("PSW");
     return("???");
 }
@@ -545,6 +579,8 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
     static regmatch_t *re_Oops_eip_ppc_pmatch;
     static regex_t     re_Oops_eip_mips;
     static regmatch_t *re_Oops_eip_mips_pmatch;
+    static regex_t     re_Oops_eip_kdb;
+    static regmatch_t *re_Oops_eip_kdb_pmatch;
     static regex_t     re_Oops_eip_other;
     static regmatch_t *re_Oops_eip_other_pmatch;
     static const char procname[] = "Oops_eip";
@@ -635,6 +671,28 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 	return((*string)[re_Oops_eip_mips.re_nsub]);
     }
 
+    /* SGI kdb backtrace EIP.  At start of line, 0x unbracketed
+     * address, 0x unbracketed address, anything, '+0x'.  The second
+     * unbracketed address is the EIP.
+     */
+    RE_COMPILE(&re_Oops_eip_kdb,
+	    "^0x" UNBRACKETED_ADDRESS	/* 1 */
+	    "0x" UNBRACKETED_ADDRESS	/* 2 */
+	    ".*+0x",
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_eip_kdb_pmatch);
+
+    i = regexec(&re_Oops_eip_kdb, line, re_Oops_eip_kdb.re_nsub+1,
+	re_Oops_eip_kdb_pmatch, 0);
+    DEBUG(4, "regexec kdb %d", i);
+    if (i == 0) {
+	re_string_check(re_Oops_eip_kdb.re_nsub+1, string_max,
+	    procname);
+	re_strings(&re_Oops_eip_kdb, line, re_Oops_eip_kdb_pmatch,
+	    string);
+	return((*string)[2]);
+    }
+
     /* Oops 'EIP:' line for other architectures */
     RE_COMPILE(&re_Oops_eip_other,
 		"^("
@@ -885,6 +943,40 @@ static void Oops_set_i370_psw(const char *value, SYMBOL_SET *ss,
     add_symbol_n(ss, PSW, 'E', 1, ">>PSW;");
 }
 
+/* Look for the s390 PSW line, returns start of the relevant hex value */
+static char *Oops_s390_psw(const char *line, char ***string, int string_max)
+{
+    int i;
+    static regex_t     re_Oops_s390_psw;
+    static regmatch_t *re_Oops_s390_psw_pmatch;
+    static const char procname[] = "Oops_s390_psw";
+
+    /* Oops 'User PSW:' line for s390, PSW at start of line, value is
+     * second address.
+     */
+    RE_COMPILE(&re_Oops_s390_psw,
+	    "^User PSW: *"
+	    UNBRACKETED_ADDRESS		/* flags */
+	    UNBRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_s390_psw_pmatch);
+
+    i = regexec(&re_Oops_s390_psw, line, re_Oops_s390_psw.re_nsub+1,
+	re_Oops_s390_psw_pmatch, 0);
+    DEBUG(4, "regexec %d", i);
+    if (i == 0) {
+	re_string_check(re_Oops_s390_psw.re_nsub+1, string_max, procname);
+	re_strings(&re_Oops_s390_psw, line, re_Oops_s390_psw_pmatch,
+	    string);
+	return((*string)[re_Oops_s390_psw.re_nsub]);
+    }
+    return(NULL);
+}
+
+/* No need for Oops_set_s390_psw, can use Oops_set_i370_psw for both */
+#define Oops_set_s390_psw(line, string, string_max) \
+        Oops_set_i370_psw(line, string, string_max)
+
 /* Look for the Trace multilines :(.  Returns start of addresses. */
 static const char *Oops_trace(const char *line)
 {
@@ -900,6 +992,11 @@ static const char *Oops_trace(const char *line)
      * bracketed address.  Alas users have been known to split lines so "(nn)"
      * appears at the start of the continuation line, look for that as well.
      * Life would be so much easier without users.
+     * The person who did the s390 trace obviously did not like any of the ways
+     * that other Linux kernels report problems so they invented a system all on
+     * their own.  It is different from the i370 trace and even more ambiguous
+     * than the other trace formats.  That is what happens when code is written
+     * in secret!
      */
     RE_COMPILE(&re_Oops_trace,
 	    "^("						     /* 1 */
@@ -907,12 +1004,13 @@ static const char *Oops_trace(const char *line)
     /* alpha */     "|(Trace: )"				     /* 3 */
     /* various */   "|(" BRACKETED_ADDRESS ")"			     /* 4,5 */
     /* ppc */	    "|(Call backtrace:)"			     /* 6 */
-    /* ppc */	    "|" UNBRACKETED_ADDRESS			     /* 7 */
+    /* ppc, s390 */ "|" UNBRACKETED_ADDRESS			     /* 7 */
     /* arm */	    "|Function entered at (" BRACKETED_ADDRESS ")"   /* 8,9 */
     /* sparc64 */   "|Caller\\[" UNBRACKETED_ADDRESS "\\]"	     /* 10 */
     /* i386 */	    "|(" REVBRACKETED_ADDRESS ")"		     /* 11,12 */
     /* ikd patch */ "|(\\([0-9]+\\) *(" BRACKETED_ADDRESS "))"	     /* 13,14,15 */
     /* i370 */	    "|([0-9]+ +base=0x" UNBRACKETED_ADDRESS ")"	     /* 16,17 */
+    /* s390 */	    "|(Kernel BackChain.*)"                          /* 18    */
 	    ")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_trace_pmatch);
@@ -948,6 +1046,11 @@ static const char *Oops_trace(const char *line)
 	    trace_line = 3;
 	    start = line + re_Oops_trace_pmatch[17].rm_so;
 	}
+	else if (MATCHED(18)) {
+	    /* s390 only line, no addresses on this line */
+	    trace_line = 4;
+	    start = line + re_Oops_trace_pmatch[18].rm_eo;
+	}
 	else if (MATCHED(11)) {
 	    /* Reversed addresses for spinloops :( */
 	    trace_line = 1;
@@ -959,8 +1062,8 @@ static const char *Oops_trace(const char *line)
 	else if (trace_line == 1 && MATCHED(14))
 	    /* incorrectly split trace lines from ikd patch */
 	    start = line + re_Oops_trace_pmatch[14].rm_so;
-	else if (trace_line == 2 && MATCHED(7))
-	    /* ppc continuation line */
+	else if ((trace_line == 2 || trace_line == 4) && MATCHED(7))
+	    /* ppc/s390 continuation line */
 	    start = line + re_Oops_trace_pmatch[7].rm_so;
 	else
 	    trace_line = 0;
@@ -991,7 +1094,7 @@ static void Oops_trace_line(const char *line, const char *p, SYMBOL_SET *ss)
 	REG_NEWLINE|REG_EXTENDED,
 	&re_ikd_pmatch);
 
-    /* ppc does not bracket its addresses */
+    /* ppc/s390 does not bracket its addresses */
     if (isxdigit(*p)) {
 	pregex = &re_unbracketed_address;
 	pregmatch = re_unbracketed_address_pmatch;
@@ -1033,7 +1136,7 @@ static const char *Oops_i370_regs(const char *line)
     static const char procname[] = "Oops_i370_regs";
 
     RE_COMPILE(&re_Oops_i370_regs,
-	    "^ *(r[0-9]{1,2}): *" UNBRACKETED_ADDRESS,	/* 1, 2 */
+	    "^(r[0-9]{1,2}): *" UNBRACKETED_ADDRESS,	/* 1, 2 */
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_i370_regs_pmatch);
 
@@ -1087,6 +1190,85 @@ static void Oops_set_i370_regs(const char *line, const char *p,
     re_strings_free(&re_Oops_i370_regs, &string);
 }
 
+/* Decide if the line contains s390 registers.  Returns first register value.
+ * This is messier than the trace lines.  The heading is on one line followed by
+ * the registers with *NO* keywords.  Fine for a dedicated JESMSG file, very
+ * poor design for a multi-user log system.
+ */
+static const char *Oops_s390_regs(const char *line)
+{
+    int i;
+    static regex_t     re_Oops_s390_regs;
+    static regmatch_t *re_Oops_s390_regs_pmatch;
+    static const char procname[] = "Oops_s390_regs";
+
+    RE_COMPILE(&re_Oops_s390_regs,
+	    "^(User GPRS.*)",
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_s390_regs_pmatch);
+
+    i = regexec(&re_Oops_s390_regs, line, re_Oops_s390_regs.re_nsub+1,
+	re_Oops_s390_regs_pmatch, 0);
+    DEBUG(4, "regexec %d", i);
+    if (i == 0) {
+	s390_reg_num = 0;  /* Start with R0 */
+	return(line + re_Oops_s390_regs_pmatch[0].rm_eo);
+    }	
+    else if (s390_reg_num >= 0) /* Header already seen */ {
+	i = regexec(&re_unbracketed_address, line,
+	    re_unbracketed_address.re_nsub+1,
+	    re_unbracketed_address_pmatch, 0);
+	DEBUG(4, "regexec %d", i);
+	if (i == 0)
+	    return(line + re_unbracketed_address_pmatch[0].rm_so);
+    }
+    s390_reg_num = -1;  /* Not processing s390 registers */	
+    return(NULL);
+}
+
+/* Process a s390 register line, extract addresses */
+static void Oops_set_s390_regs(const char *line, const char *p,
+			       SYMBOL_SET *ss, const OPTIONS *options)
+{
+    addr_t reg;
+    char regname[7];
+    char **string = NULL;
+    static const char procname[] = "Oops_set_s390_regs";
+
+    /* Loop over s390 unbracketed register values.  No keywords so we have to
+     * synthesize the register numbers ourselves, yuck!
+     */
+    while (1) {
+	if (regexec(&re_unbracketed_address, p,
+	    re_unbracketed_address.re_nsub+1,
+	    re_unbracketed_address_pmatch, 0) == 0) {
+	    re_strings(&re_unbracketed_address, p,
+		re_unbracketed_address_pmatch, &string);
+	    reg = hexstring(string[1]);		/* contents */
+	    if (errno) {
+		ERROR(" Invalid hex value in s390 register line, ignored - '%s'",
+		      string[1]);
+		perror(prefix);
+		reg = 0;
+	    }
+	    reg = Oops_truncate_address(reg, options);
+	    if (reg > 1024) {
+		strcpy(regname, ">>");
+		snprintf(regname, sizeof(regname), "r%-2d;  ", s390_reg_num);
+		add_symbol_n(ss, reg, 'R', 1, regname);
+	    }
+	    p += re_unbracketed_address_pmatch[0].rm_eo;
+	    ++s390_reg_num;
+	}
+	else
+	    break;
+    }
+
+    if (*p)
+	WARNING("garbage '%s' at end of s390 register line ignored", p);
+    re_strings_free(&re_unbracketed_address, &string);
+}
+
 /* Do pattern matching to decide if the line should be printed.  When reading a
  * syslog containing multiple Oops, you need the intermediate data (registers,
  * tss etc.) to go with the decoded text.  Sets text to the start of the useful
@@ -1223,6 +1405,13 @@ static int Oops_print(const char *line, const char **text, char ***string,
      * of multiple lines, each containing a number (trace depth), base=, link=,
      * stack=.
      *
+     * s390 decided to do their own thing instead of using the same format as
+     * i370.  A bad case of "not invented here".  s390 data starts on a separate
+     * line and consists of multiple addresses.
+     *
+     * The SGI kdb backtrace for ix86 prints EBP and EIP, one per line after the
+     * header.
+     *
      * Some strings indicate stack data (S), some indicate trace data (T).
      */
     RE_COMPILE(&re_Oops_print_s,
@@ -1230,7 +1419,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
 		"^("					/*  1 */
     /* i386 */	    "(Stack: )"				/*  2  S */
     /* m68k */	    "|(Stack from )"			/*  3  S */
-    /* various */   "|([0-9a-fA-F]{4,})"		/*  4  S */
+    /* various */   "|([0-9a-fA-F]{4,})"		/*  4  S,T */
     /* various */   "|(Call Trace: )"			/*  5  T */
     /* various */   "|(" BRACKETED_ADDRESS ")"		/* 6,7 T */
     /* various */   "|(Version_[0-9]+)"			/*  8 */
@@ -1241,6 +1430,9 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* ikd/AK */    "|(\\([^ ]+\\) *"			/* 13  T */
 		      BRACKETED_ADDRESS ")"		/* 14    */
     /* i370 */	    "|([0-9]+ +base=)"			/* 15  T */
+    /* s390 */	    "|(Kernel BackChain)"		/* 16  T */
+    /* kdb ix86 */  "|EBP *EIP"				/* 17  T */
+    /* kdb ix86 */  "|0x" UNBRACKETED_ADDRESS "0x" UNBRACKETED_ADDRESS	/* 18, 19 T */
 
 
     /* order does not matter from here on */
@@ -1341,6 +1533,13 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* i370 */	    "|Kernel stack "
     /* any other i370 lines to print? */
 
+    /* s390 */	    "|User PSW"
+    /* s390 */	    "|User GPRS"
+    /* s390 */	    "|User ACRS"
+    /* s390 */	    "|illegal operation"
+    /* s390 */	    "|task:"
+    /* any other s390 lines to print? */
+
 		")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_print_s_pmatch);
@@ -1356,6 +1555,9 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* Handle multiline messages, messy.  Only treat strings that look like
      * addresses as useful when they immediately follow a definite stack or
      * trace header or a previously valid stack or trace line.
+     * Because of the horribly ambiguous s390 register dump, unbracketed
+     * addresses (string 4) can now be either stack or trace.  How not to design
+     * dump output.
      */
     /* Stack lines */
     if (!MATCHED(2) && !MATCHED(3) && !MATCHED(4))
@@ -1365,19 +1567,21 @@ static int Oops_print(const char *line, const char **text, char ***string,
     else if (stack_line && !MATCHED(4))
 	stack_line = 0;
     /* Trace lines */
-    if (!MATCHED(5) && !MATCHED(6) && !MATCHED(9) && !MATCHED(10) &&
-	!MATCHED(11) && !MATCHED(12) && !MATCHED(13) && !MATCHED(15))
+    if (!MATCHED(4) && !MATCHED(5) && !MATCHED(6) && !MATCHED(9) &&
+        !MATCHED(10) && !MATCHED(11) && !MATCHED(12) && !MATCHED(13) &&
+	!MATCHED(15) && !MATCHED(16) && !MATCHED(17) && !MATCHED(19))
 	trace_line = 0;
-    else if (MATCHED(5) || MATCHED(9) || MATCHED(10) || MATCHED(11) ||
-             MATCHED(15))
+    else if (MATCHED(4) || MATCHED(5) || MATCHED(9) || MATCHED(10) ||
+        MATCHED(11) || MATCHED(15) || MATCHED(16) || MATCHED(17))
 	trace_line = 1;
-    else if (trace_line && !MATCHED(6) && !MATCHED(12) && !MATCHED(13))
+    else if (trace_line && !MATCHED(4) && !MATCHED(6) && !MATCHED(12) &&
+        !MATCHED(13) && !MATCHED(19))
 	trace_line = 0;
     if (i == 0) {
 	print = 1;
-	if (MATCHED(4) && !stack_line) {
+	if (MATCHED(4) && !stack_line && !trace_line) {
 	    print = 0;
-	    DEBUG(4, "ambiguous stack line ignored");
+	    DEBUG(4, "ambiguous stack/trace line ignored");
 	}
 	if ((MATCHED(6) || MATCHED(12) || MATCHED(13)) && !trace_line) {
 	    print = 0;
@@ -1493,6 +1697,7 @@ static const char *Oops_code(const char *line, char ***string, int string_max)
 		": +"
 		"("				/*  4 */
 		  "(general protection.*)"
+		  "|(Bad EIP value.*)"
 		  "|(<[0-9]+>)"
 		  "|(([<(]?[0-9a-fA-F]+[>)]? +)+[<(]?[0-9a-fA-F]+[>)]?)"
 		")"
@@ -1544,8 +1749,7 @@ static void Oops_decode(const unsigned char* code_text, addr_t eip,
 
     DEBUG(1, "");
     /* text to binary */
-    if (!Oops_code_values(code_text, code, &adjust, string, string_max,
-			  options))
+    if (!Oops_code_values(code_text, code, &adjust, options))
 	return;
     /* binary to same format as ksymoops */
     if (!(file = Oops_code_to_file(code, CODE_SIZE, ibfd, options)))
@@ -1722,10 +1926,14 @@ int Oops_read(OPTIONS *options)
 		    Oops_set_sparc_oi7(start, &string, &ss_format);
 		if ((start = Oops_i370_psw(text, &string, MAX_STRINGS)))
 		    Oops_set_i370_psw(start, &ss_format, options);
+		if ((start = Oops_s390_psw(text, &string, MAX_STRINGS)))
+		    Oops_set_s390_psw(start, &ss_format, options);
 		if ((start = Oops_trace(text)))
 		    Oops_trace_line(text, start, &ss_format);
 		if ((start = Oops_i370_regs(text)))
 		    Oops_set_i370_regs(text, start, &ss_format, options);
+		if ((start = Oops_s390_regs(text)))
+		    Oops_set_s390_regs(text, start, &ss_format, options);
 		if ((start = Oops_code(text, &string, MAX_STRINGS))) {
 		    /* In case no EIP line was seen */
 		    Oops_set_default_ta(me, ibfd, options);
